program mksrclist

  use dal
  use constants
!  use sla
  use coordinate

  implicit none

  ! Fake pointing direction
  real (kind=double), parameter :: pntRADeg   = 193.18543957
  real (kind=double), parameter :: pntDecDeg  = -18.235478234
  real (kind=double), parameter :: pntAposDeg =  32.331659821

  ! Some sizes (in arcseconds)
  real (kind=single), parameter :: fieldOfView=13.5*60
  real (kind=single), parameter :: skyPixelSize=0.05

  ! How many?
  integer, parameter :: nSrc=400

  ! Handles
  type(DataSetT) :: set
  type(TableT) :: tab
  type(ColumnT) :: racol, deccol, azcol, elcol, rateCol, bkgMapCol, countsCol
  type(ColumnT) :: idInstCol, idBandCol

  ! Pointers, etc.
  integer :: i
  real (single), pointer, dimension(:) :: rate, bkgRate, countsData
  real (double), pointer, dimension(:) :: raDeg, decDeg, az, el
  integer(int32), pointer, dimension(:) :: idInst, idBand

  !
  call random_seed()

  ! Make source list with lots of sources
  call create_srclist('srclist.fits',400)

  ! Make source list with one source
  call create_srclist('onesource.fits',1)

contains

  !---------------------------------------------------------------------
  subroutine create_srclist(name,nSrc)
  !---------------------------------------------------------------------

    implicit none

    character (len=*), intent(in) :: name
    integer, intent(in) :: nSrc
real(double) :: ra, dec

    !-------------------------------------------------------------------
    ! Start

    ! Create dataset
    set=dataSet(name,create)
    tab=addTable(set,"SRCLIST",nSrc)

    !
    call setAttribute(set,"RA_PNT",pntRADeg)
    call setAttribute(set,"DEC_PNT",pntDecDeg)
    call setAttribute(set,"PA_PNT",pntAposDeg)

    call setAttribute(set,"TCRPIX1",0)
    call setAttribute(set,"TCRPIX2",0)
    call setAttribute(set,"TCDELT1",-skyPixelSize/3600.0)
    call setAttribute(set,"TCDELT2",-skyPixelSize/3600.0)
    call setAttribute(set,"TCRVAL1",pntRADeg)
    call setAttribute(set,"TCRVAL2",pntDecDeg)
    call setAttribute(set,"TCTYPE1","RA---TAN")
    call setAttribute(set,"TCTYPE2","DEC--TAN")

    ! Add columns
    idInstCol = addColumn(tab, "ID_INST", INTEGER32)
    idBandCol = addColumn(tab, "ID_BAND", INTEGER32)

    racol=addColumn(tab,"RA",Real64,"deg")
    deccol=addColumn(tab,"DEC",Real64,"deg")
    azcol=addColumn(tab,"AZ",Real64,"0.05 arcsec")
    elcol=addColumn(tab,"EL",Real64,"0.05 arcsec")
    rateCol   = addColumn(tab, "RATE",   Real32)
    bkgMapCol = addColumn(tab, "BG_MAP", Real32)
    countsCol = addColumn(tab, "SCTS",   Real32)

    idInst => int32Data(idInstCol)
    idBand => int32Data(idBandCol)
    idInst = 2
    idBand = 0

    raDeg=>real64Data(racol)
    decDeg=>real64Data(deccol)
    az=>real64Data(azcol)
    el=>real64Data(elcol)
    rate    => real32Data(rateCol)
    bkgRate => real32Data(bkgMapCol)
    countsData     => real32Data(countsCol)

    ! Populate
    call random_number(az)
    call random_number(el)
    az=(az-0.5)*2.0*fieldOfView*ArcsecToRad
    el=(el-0.5)*2.0*fieldOfView*ArcsecToRad
    do i=0,nSrc-1
!       call sla_dtp2s(az(i), el(i), pntRA*DegToRad, pntDec*DegToRad,&
!            & ra(i), dec(i))
!call tanToRaDec(pntRA*DegToRad, pntDec*DegToRad, az(i), el(i), ra(i), dec(i))
!call tanToRaDec(pntRADeg, pntDecDeg, az(i), el(i), raDeg(i), decDeg(i))
call tanToRaDec(az(i), el(i), pntRADeg*PiOn180, pntDecDeg*PiOn180, ra, dec)
! in ssclib/coordinate.
raDeg(i)  = ra  / PiOn180
decDeg(i) = dec / PiOn180
    end do
!    ra=ra*RadToDeg
!    dec=dec*RadToDeg
    az=az*RadToArcsec/skyPixelSize
    el=el*RadToArcsec/skyPixelSize
    bkgRate = 0.002
    call random_number(rate)
    rate = rate * 0.15
    countsData = rate

    ! Release handles
!    call release(racol)
!    call release(deccol)
!    call release(azcol)
!    call release(elcol)
!    call release(tab)
    call release(set)

  end subroutine create_srclist

end program mksrclist
